/* 
 *  CCDPLOT: display an image in contour/gray-scale format 
 *
 *	30-jun-87  V1.0 created from CCD as separate module		PJT
 * 	 6-jul-87  V1.2 (does read image(5) files, order keywords optimized for cnt/gray  PJT
 *	 7-jul-87  V1.3 allow matrix size and plot size different
 *	 8-jul-87  V1.4 proper cell position definition (in the middle) PJT
 *	 1-jun-88  V2.0 rename programname look -> ccdplot	PJT
 *			also new filestruct, although code same
 *	31-jan-88  V2.1 axes names from header  PJT
 *			added headline= keyword and 
 *	 1-nov-90  V2.2 drange -> nemoindp, helpvec
 *      12-jan-92  V2.3 greyscale level info in plot header     PJT
 *	23-aug-92  V2.4 new version of contour() with lineto()  PJT
 *	 6-may-95  V2.5 minor fix
 *	22-feb-97  V2.5a support for SINGLEPREC			pjt
 *	
 */

/* TO DO:
 *
 *	sort input contours before plotting, now they have to be in order
 *	If used on a cube, can only plot first plane
 *
 * BUGS:
 *	plotting subregions using xrange,yrange does not work
 */
  
#include <stdinc.h>		/* also gets <stdio.h	*/
#include <getparam.h>
#include <vectmath.h>		/* otherwise NDIM undefined */
#include <filestruct.h>
#include <image.h>
#include <yapp.h>
#include <axis.h>

string defv[] = {
	"in=???\n	input filename ",
	"contour=\n	contour levels ",
	"gray=\n	have gray scale ? (t/f) ",
	"mmin=\n	plot min for grayscale",
	"mmax=\n	plot max for grayscale [autoscale]",
	"power=1.0\n	power of transfer function ",
	"xrange=\n	range in x to plot -- BUG -- don't use ",
	"yrange=\n	range in y to plot -- BUG -- don't use ",
	"headline=\n	optional header above plot ",
	"tab=\n		table of contour segments",
	"format=%g\n	Format for above table",
	"VERSION=2.5a\n	22-feb-97 PJT",
	NULL,
};

string usage = "display an image using contours and/or gray-scales";

#define HPI  1.5702
#define RPD (3.1415/360.0)
#define HUGE 1.0e10
#define UNDEF HUGE

string	infile;				/* input file */
stream  instr;
stream  tabstr=NULL;

real    mmin,mmax;			/* maximum in map */

bool    gray;
real    power;


imageptr iptr=NULL;			/* will be allocated dynamically */
int    nx,ny,nsize;			/* size of matrix as allocated "" */
real   origin[2];			/* from header: in sky coordinates */
real xsize,ysize,size;		/* frame dimension */
real xmin,ymin,dx,dy,cell;		/* frame dimensions */

#define MCNTVAL	100			/* maximum contour levels */
string cntstr;				/* string of contour levels */
real cntval[MCNTVAL];			/* contour levels */
int ncntval;				/* actual number of contours */

real xplot[2],   yplot[2];		/* ranges in plot variables */

char   xlabel[80], ylabel[80];              
char   plabel[80], clabel[80], glabel[80];
string headline;			/* extra label */
char   format4[100];

local real xtrans(real), ytrans(real);
void lineto(real, real, real, real);

extern int contour(real*, int, int, real*, int, real, real, real, real, proc);


nemo_main()
{
    setparams();                    /* set globals */

    instr = stropen (infile, "r");
    read_image(instr,&iptr);
    strclose(instr);

    nx=Nx(iptr);			/* get stuff from header of image */
    ny=Ny(iptr);
    xmin=Xmin(iptr);
    ymin=Ymin(iptr);
    dx=Dx(iptr);
    dy=Dy(iptr);

    xsize = nx * dx;
    ysize = ny * dy;

    plinit ("***", 0.0, 20.0, 0.0, 20.0);       /* init yapp */
    plot_map();                                 /* plot the map */
    plstop();                                   /* end of yapp */
}

setparams()
{
	string  tmpstr;
	int	tmpint, i;

	infile = getparam ("in");

	cntstr = getparam ("contour");	/* get contour levels */
	if (*cntstr==0) {
/*	    ncntval = 0;		/* this will cause a grayscale plot */
	    gray=TRUE;
	} else {
	    ncntval = nemoinpr(getparam("contour"),cntval,MCNTVAL);
	    tmpstr = getparam("gray");
	    if (*tmpstr==0)
	    	gray=FALSE;
	    else
	    	gray=getbparam("gray");
	}

	if (hasvalue("mmin"))
		mmin = getdparam("mmin");	
	else
		mmin = UNDEF;

	if (hasvalue("mmax"))
		mmax = getdparam("mmax");
	else
		mmax= UNDEF;
		
	power=getdparam("power");

	if (hasvalue("xrange"))
		setrange(xplot,getparam("xrange"));
	else
		xplot[0] = xplot[1] = UNDEF;
		
	if (hasvalue("yrange"))
		setrange(yplot,getparam("yrange"));
	else
		yplot[0] = yplot[1] = UNDEF;
			
        headline = getparam("headline");
        if (hasvalue("tab")) {
            tabstr = stropen(getparam("tab"),"w");
	    tmpstr = getparam("format");
            sprintf(format4,"%s %s %s %s\n",tmpstr,tmpstr,tmpstr,tmpstr);
	}
}

plot_map ()
{
    real m_range, brightness, dcm;
    real m_min, m_max;
    int    i, ix, iy;
    int    cnt;		/* counter of pixels */

    nsize = Nx(iptr);  /* old method forced square .. */			
    cell = Dx(iptr);   /* and forced so for gray scale due to LW mem-problems */
    size = nsize*cell;
    
    m_min = MapMin(iptr);	/* get min- and max from header */
    m_max = MapMax(iptr);
    dprintf (1,"Min and max in map from map-header are: %f %f\n",m_min,m_max);
 
    if (mmax==UNDEF)	/* reset default autoscales to user supplied if necessary */
	mmax=m_max;
    if (mmin==UNDEF)
    	mmin=m_min;
    m_range = mmax-mmin;
    if (m_range==0) {
    	mmax=mmin+1.0;
	if (gray) warning("%g; Plot-range was zero, mmax increased by 1",mmin);
    }
        
    dprintf (1,"User reset Min and max are: %f %f\n",mmin,mmax);

    
    sprintf (plabel,"File: %s",infile);			/* filename */
    sprintf (clabel,"Contours: %s",cntstr);             /* contour levels */
    sprintf (glabel,"Gray MinMax: %g %g",mmin,mmax);    /* grey scale minmax */

	/* set scales and labels along axes */
    if (xplot[0]==UNDEF || xplot[1]==UNDEF) {
    	xplot[0] = Xmin(iptr) - 0.5*Dx(iptr);
    	xplot[1] = xplot[0] + Nx(iptr)*Dx(iptr);
    }
    if (Namex(iptr))
        strncpy(xlabel,Namex(iptr),80);
    else
        strcpy (xlabel,"");

    if (yplot[0]==UNDEF || yplot[1]==UNDEF) {
    	yplot[0] = Ymin(iptr) - 0.5*Dy(iptr);
    	yplot[1] = yplot[0] + Ny(iptr)*Dy(iptr);
    }
    if (Namey(iptr))
        strncpy(ylabel,Namey(iptr),80);
    else
        strcpy (ylabel,"");
	
    dprintf (1,"Plotting area x=%f:%f y=%f:%f\n",
    	xplot[0], xplot[1], yplot[0], yplot[1]);

    if (gray) {		
				/* gray-scale */
       dcm = Dx(iptr) / (xplot[1]-xplot[0]) * 16.0;
       pl_matrix (Frame(iptr), nx, ny, 
           xtrans(Xmin(iptr)), ytrans(Ymin(iptr)), dcm , mmin, mmax, power);
/*       color_bar (100.0,500.0,32);	*/
     } 
     		/* OLD ROUTINE, has to call relocate/frame ---> plcontour */
     contour (Frame(iptr),nx,ny,cntval,ncntval,
		Xmin(iptr), 
		Ymin(iptr), 
		Xmin(iptr)+(Nx(iptr)-1)*Dx(iptr),
		Ymin(iptr)+(Ny(iptr)-1)*Dy(iptr), lineto);

	/* draw axes and their labels */
    xaxis ( 2.0, 2.0, 16.0, xplot, -7, xtrans,  xlabel);
    xaxis ( 2.0,18.0, 16.0, xplot, -7, xtrans,  NULL);
    yaxis ( 2.0, 2.0, 16.0, yplot, -7, ytrans, ylabel);
    yaxis (18.0, 2.0, 16.0, yplot, -7, ytrans, NULL);

    pltext (plabel,2.0,18.4, 0.32, 0.0);	/* plot header with file name */
    pltext (clabel,2.0,19.0, 0.32, 0.0);        /* plot header with contour levels */
    pltext (glabel,2.0,19.6, 0.32, 0.0);        /* plot header with greyscale info */
    pljust(1);
    pltext (headline,10.0,18.4, 0.26, 0.0);     /* plot extra user suplied header */
    pljust(-1);
}

#if 0
color_bar(xmin, xmax, ncolors)	/* should go to yapp */
real xmin, xmax;
int ncolors;
{
	int i;
	real graylevel, x;
	char line[200];

	dx = (xmax-xmin)/ncolors;
	x = xmin;
	graylevel = 0.0;
	for (i=0; i<ncolors; i++) {

		vec_paint (x,50.0,dx,graylevel);
		x += dx;
	}
	sprintf (line,"%.1f %.1f mt %.1f %.1f lt s",xmin,45.0,xmax,45.0);
	vec_write (line);
}
#endif

/*********** SOME LOCAL UTILITIES , PLOTTING ANDSO ***********/
/* some functions for yapp-plotting */

local real xtrans(real x)
{
    return 2.0 + 16.0*(x-xplot[0])/(xplot[1]-xplot[0]) ;
}


local real ytrans(real y)	/* upper panel: mass surface density */
{
    return 2.0 + 16.0*(y-yplot[0])/(yplot[1]-yplot[0]) ;
}

void lineto(real x1, real y1, real x2, real y2)
{
    if (tabstr) fprintf(tabstr,format4,x1,y1,x2,y2);

    plmove (xtrans(x1),ytrans(y1));
    plline (xtrans(x2),ytrans(y2));
}
		
/*******  stolen from snapplot.c **********/

setrange(real *rval, string rexp)
{
    char  *cptr;
	
    cptr = strchr(rexp, ':');
    if (cptr != NULL) {
        rval[0] = atof(rexp);
        rval[1] = atof(cptr+1);
    } else {
        rval[0] = 0.0;
        rval[1] = atof(rexp);
    } 
}
					
					
