/* 
 *  FTOC:
 *      1)  extract a defv[] string array for a nemo_main()
 *          this is useful to be able to call C from Fortran
 *          and add the NEMO user interface
 *
 *	27-jun-91 recreated...		PJT
 *	24-may-92 added a 'c:' line as usage line (also conform MIRIAD)  PJT
 *      20-may-94 added hpux to not convert fortran external name
 */

#include <stdinc.h>
#include <getparam.h>
#include <ctype.h>

string defv[] = {
    "in=???\n           Input source code to extract defv[] from",
    "out=-\n            Output nemo_main module [- means stdout]",
    "call=nemomain\n    You fortran main subroutine (no args!)",
    "machine=\n         machine for symbols name conversions",
    "VERSION=1.2\n      20-may-94 PJT",
    NULL,
};

string usage="nemo_main() builder from fortran source code";
string e_usage=NULL;

#define MAXLINE 256

nemo_main()
{
    stream instr, outstr;
    bool notdone=TRUE, process();
    char line[MAXLINE+1];

    instr = stropen(getparam("in"),"r");
    outstr = stropen(getparam("out"),"w");

    header(outstr);
    while(notdone) {
        if (fgets(line,MAXLINE,instr)==NULL)
            break;
        notdone = process(outstr,line);
    }
    if (notdone)
        error("Input file does not have properly closed defv section");
    strclose(instr);
    footer(outstr,getparam("call"),getparam("machine"));
    strclose(outstr);
}

header(outstr)
stream outstr;
{
    fprintf(outstr,"/* THIS FILE HAS BEEN CREATED BY %s - do not edit */\n\n",
            getargv0());
    fprintf(outstr,"#include <stdinc.h>\n\n");
    fprintf(outstr,"string defv[] = {\n");
}

footer(outstr,callname,machname)
stream outstr;
string callname, machname;
{
    char name[MAXLINE];

    if (*callname)
        strcpy(name,callname);
    else
        strcpy(name,"nemomain");
    mksymname(name, machname);
    
    fprintf(outstr,"    NULL,\n};\n\n");
    fprintf(outstr,"string usage=\"%s\";\n\n",
        (e_usage ? e_usage : "NEMO program with unknown intent"));
    fprintf(outstr,"nemo_main()\n{\n");
    fprintf(outstr,"    %s();\n}\n",name);
}


mksymname(name, machname)
char *name, *machname;
{
    char *cp=name;
    int upper=0, underend=0;

    if (name==NULL || *name==NULL) return(0);

    if (machname==NULL || *machname == NULL) {
#if defined(unicos) || defined(cray) || defined(cray2)
       upper=1;
#elif defined(hpux)
       ;
#else
       underend=1;
#endif    
    } else {
        if (streq(machname,"cray"))
            upper=1;
        else if (streq(machname,"vms") || streq(machname,"hpux"))
	    ;       /* do nothing */
	else if (streq(machname,"bsd"))
            underend=1;
        else {
            warning("MKSYSNAME: Machine type %s not uderstood, bsd assumed",
                    machname);
            underend=1;
        }

    }

    if (upper) {            /* convert to upper case */
        while (*cp) {
            if (islower(*cp))
                *cp = toupper(*cp);
            cp++;
        }
    }

    if (underend) {         /* append underscore */
        while (*cp) 
            cp++;
        *cp++ = '_';
        *cp = '\0';
    }
}

bool process(outstr,line)
stream outstr;
char *line;
{
    char *cp, *scopy();
    static int doc_status = -1;     /* -1: no started 0:inside   1: done */
    static int lineno=0;

    lineno++;
    cp =line;

    if (*cp != 'c' && *cp != 'C') return TRUE;
    cp++;
    if (*cp == ':') {
        cp++;
        while (*cp == ' ' || *cp == '\t') cp++;     /* skipwhite */
        if (*cp==NULL || *cp=='\n') return TRUE;
        if (cp[strlen(cp)-1] == '\n') /* patch possible newline from fgets() */
            cp[strlen(cp)-1] = '\0';
        e_usage = scopy(cp);
        return TRUE;
    }
    if (*cp == '+') {
        if (doc_status != -1) error("Illegal c+ at line %d\n",lineno);
        doc_status = 0;
        return TRUE;
    }
    if (*cp == '-') {
        if (doc_status != 0)  error("Illegal c- at line %d\n",lineno);
        return FALSE;
    }
    if (doc_status != 0) return TRUE;
    cp++;
    while (*cp == ' ' || *cp == '\t') cp++;  /* skipwhite */
    if (*cp == NULL)                    /* skip 'blank' lines */
        return TRUE;
    if (cp[strlen(cp)-1] == '\n')       /* patch possible newline from fgets() */
        cp[strlen(cp)-1] = '\0';
    fprintf(outstr,"    \"%s\",\n",cp);
    return TRUE;
}
