/*
 *  MAIN.C: NEMO driver for nbody2
 *	
 *	16-jun-97   V1.0    does the basics, nothing fancy      PJT
 *      18-jun-97   V1.1    more logic builtin (see input.f, data.f, scale.f)
 *	21-jun-97   V1.1a   added history
 *	15-jul-97   V1.2    changed out= to outdir=, no def for nbody=     PJT
 *                          added tnext=
 *
 */

#include <stdinc.h>
#include <getparam.h>

#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

string defv[] = {
    "in=\n            input file (optional - see nbody= ) ",
    "outdir=???\n     output run directory (required)",

    "nbody=???\n      Total particle number (<= NMAX).",
    "nfix=1\n         Output frequency of data save or binaries; KZ(3 & 6)",
    "nrand=0\n        Random number sequence skip",
    "nnbmax=40\n      Maximum number of neighbours (< LMAX)",
    "nrun=1\n         Run identification index",

    "etai=0.02\n      Time-step parameter for irregular force polynomial",
    "etar=0.04\n      Time-step parameter for regular force polynomial",
    "rs0=1.0\n        Initial radius of neighbour sphere",
    "deltat=0.5\n     Output time interval in units of the crossing time",
    "tcrit=5.0\n      Termination time in units of the crossing time",
    "tnext=\n         Next output time for restart runs",
    "qe=0.00002\n     Energy tolerance (restart if DE/E > 5*QE & KZ(2) > 1)",
    "eps=0.01\n       Potential softening parameter",


    "kz=1,2,0,0,0,0,1,1,0,0,1\n  Non-zero options for alternative paths (see below)\n"
      " 1  COMMON save on unit 1 at end of run (=2: every 100*NMAX steps)\n"
      " 2  COMMON save on unit 2 at output (=1); restart if DE/E > 5*QE (=2)\n"
      " 3  Basic data written to unit 3 at output time (frequency NFIX)\n"
      " 4  Initial conditions on unit 4 (=1: output; =2: input)\n"
      " 5  Initial conditions (=0: uniform & isotropic; =1: Plummer model)\n"
      " 6  Output of significant binaries (=2: frequency NFIX)\n"
      " 7  Lagrangian radii (=1: unit 6; =2: unit 7; =3: both types)\n"
      " 8  Core radius & density centre (N > 20 only)\n"
      " 9  Individual bodies printed at output time (MIN(5**KZ9,N))\n"
     " 10  No unique density centre (skips velocity modification of RS(I))\n"
     " 11  Modification of ETAI & ETAR by tolerance QE\n"
     " 12  Inelastic mergers (>1: diagnostic output)\n"
     " 13  Escaper removal (R > 2*RTIDE; RTIDE = 10*RSCALE)\n"
     " 14  Skip full predictor loop if NNB > KZ(14) = <NNB> & KZ(14) > 0\n"
     " 15  External potential (=1: Plummer model; =2: logarithmic potential)\n"
     " 16  No scaling of initial conditions\n"
     " 17  Generation of two subsystems (merger experiment)\n"
     " 18  Adjustment of coordinates & velocities to c.m. condition\n"
     " 19  Not used at present (same for # 20)",

    "xtpar1=\n        Mass of external Plummer model (KZ(15) = 1; scaled units)",
    "xtpar2=\n        Length scale for Plummer model (KZ(15) = 1)",
    "zmgas=\n         Mass scale for logarithmic potential (KZ(15) = 2)",
    "rgas=\n          Length scale for logarithmic potential (KZ(15) = 2)",

    "alphas=2.3\n     Power-law index for initial mass function",
    "body1=5.0\n      Maximum particle mass before scaling",
    "bodyn=1.0\n      Minimum particle mass before scaling",

    "q=0.0\n          Virial ratio (q=0.5 for virial equilibrium)",
    "vxrot=0.0\n      XY-velocity scaling factor (> 0 for solid-body rotation)",
    "vzrot=0.0\n      Z-velocity scaling factor (not used if VXROT = 0)",
    "rbar=1.0\n       Virial radius in pc (for scaling to physical units)",
    "zmbar=1.0\n      Mean mass in solar units",

    "xcm=\n           Displacement for subsystem (routine SCALE; KZ 17)",
    "ecc=\n           Eccentricity of relative motion for subsystem (ECC =< 1)",

    "kstart=1\n       Running mode (1=new 2=restart 3,4,5=restart w/ new par",
    "tcomp=40.0\n     Maximum allowed running time (minutes)",

    "VERSION=1.2a\n   1-aug-97 PJT",
    NULL,
};

string usage="Ahmad-Cohen N-body code";

nemo_main()
{
    int nbody, nfix, nrand, nnbmax, nrun, kstart;
    real etai, etar, rs0, deltat, tcrit, qe, eps, tcomp;
    int k, nkz, kz[20];
    real alphas, body1, bodyn;
    real q, vxrot, vzrot, rbar, zmbar;
    string exefile = "nbody2";
    string parfile = "nbody2.in";
    string rundir = getparam("outdir");
    string infile;
    char fullname[256], runcmd[256];
    stream datstr, histr;

    kstart = getiparam("kstart");
    tcomp =  getdparam("tcomp");

    nbody = getiparam("nbody");
    nfix = getiparam("nfix");
    nrand = getiparam("nrand");
    nnbmax = getiparam("nnbmax");
    nrun = getiparam("nrun");

    etai = getdparam("etai");
    etar = getdparam("etar");
    rs0 = getdparam("rs0");
    deltat = getdparam("deltat");
    tcrit = getdparam("tcrit");
    qe = getdparam("qe");
    eps = getdparam("eps");

    nkz = nemoinpi(getparam("kz"),kz,20);
    if (nkz < 0) error("%d Syntax error kz=%s",nkz,getparam("kz"));
    for (k=nkz; k<20; k++) kz[k]=0;
    for (k=0; k<20; k++) dprintf(1,"%d ",kz[k]);
    dprintf(1,"\n");

    alphas = getdparam("alphas");
    body1 = getdparam("body1");
    bodyn = getdparam("bodyn");

    q = getdparam("q");
    vxrot = getdparam("vxrot");
    vzrot = getdparam("vzrot");
    rbar = getdparam("rbar");
    zmbar = getdparam("zmbar");

    make_rundir(rundir);

    sprintf(fullname,"%s/%s",rundir,parfile);
    datstr = stropen(fullname,"w");    

    /*  New Run */

  if (kstart == 1) {

    fprintf(datstr,"%d %g\n",kstart,tcomp);
    fprintf(datstr,"%d %d %d %d %d\n",nbody,nfix,nrand,nnbmax,nrun);
    fprintf(datstr,"%g %g %g %g %g %g %g\n",etai,etar,rs0,deltat,tcrit,qe,eps);

    for (k=0; k<20; k++) fprintf(datstr,"%d ",kz[k]);
    fprintf(datstr,"\n");

    if (kz[14] > 0) {           /* external potential parameters */
        if (kz[14]==1)
            fprintf(datstr,"%g %g\n",getdparam("xtpar1"),getdparam("xtpar2"));
        else
            fprintf(datstr,"%g %g\n",getdparam("zmgas"),getdparam("rgas"));
    }
            
    fprintf(datstr,"%g %g %g\n",alphas,body1,bodyn);
    fprintf(datstr,"%g %g %g %g %g\n",q,vxrot,vzrot,rbar,zmbar);

    if (kz[16] > 0)
        fprintf(datstr,"%g %g\n",getdparam("xcm"), getdparam("ecc"));
    strclose(datstr);

    goto_rundir(rundir);
    histr = stropen("history","w");
    put_history(histr);
    strclose(histr);

    if (hasvalue("in")) {
	infile = getparam("in");
	if (*infile == '-') {		/* do something special for pipes */
            sprintf(runcmd,"stou4 %s",infile);
	} else if (*infile == '/') {	/* regular file */
            sprintf(runcmd,"stou4 %s nbody=%d",infile,nbody);
        } else
            error("in=%s must be an absolute pathname or special file",infile);
        dprintf(0,"%s\n",runcmd);
        if (system(runcmd)) error("Error converting input data");
    }

    sprintf(runcmd,"%s < %s",exefile,parfile);
    run_program(runcmd);
  } else {
    error("kstart=%d not yet supported",kstart);
  }
}


goto_rundir(string name)
{
    if (chdir(name))
        error("Cannot change directory to %s",name);
}

make_rundir(string name)
{
    if (mkdir(name, 0755))
        warning("Run directory %s already exists",name);
}

run_program(string cmd)
{
    system(cmd);
}
